package javax.swing;

import javax.accessibility.AccessibleIcon;
import java.awt.event.FocusListener;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Font;
import java.awt.Cursor;
import java.util.Locale;
import javax.accessibility.AccessibleStateSet;
import javax.accessibility.AccessibleComponent;
import java.awt.Point;
import javax.accessibility.AccessibleRole;
import javax.accessibility.AccessibleSelection;
import javax.accessibility.AccessibleContext;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.awt.event.MouseEvent;
import java.awt.Rectangle;
import java.awt.Color;
import javax.accessibility.AccessibleState;
import java.beans.Transient;
import sun.swing.SwingUtilities2;
import javax.swing.plaf.UIResource;
import javax.swing.plaf.TabbedPaneUI;
import java.util.ArrayList;
import javax.swing.event.ChangeEvent;
import java.awt.Component;
import java.util.List;
import javax.swing.event.ChangeListener;
import javax.accessibility.Accessible;
import java.io.Serializable;

public class JTabbedPane extends JComponent implements Serializable, Accessible, SwingConstants
{
    public static final int WRAP_TAB_LAYOUT = 0;
    public static final int SCROLL_TAB_LAYOUT = 1;
    private static final String uiClassID = "TabbedPaneUI";
    protected int tabPlacement;
    private int tabLayoutPolicy;
    protected SingleSelectionModel model;
    private boolean haveRegistered;
    protected ChangeListener changeListener;
    private final List<JTabbedPane.Page> pages;
    private Component visComp;
    protected transient ChangeEvent changeEvent;
    
    public JTabbedPane() {
        this(1, 0);
    }
    
    public JTabbedPane(int n) {
        this(n, 0);
    }
    
    public JTabbedPane(int tabPlacement, int tabLayoutPolicy) {
        this.tabPlacement = 1;
        this.changeListener = null;
        this.visComp = null;
        this.changeEvent = null;
        this.setTabPlacement(tabPlacement);
        this.setTabLayoutPolicy(tabLayoutPolicy);
        this.pages = new ArrayList(1);
        this.setModel(new DefaultSingleSelectionModel());
        this.updateUI();
    }
    
    public TabbedPaneUI getUI() {
        return (TabbedPaneUI)this.ui;
    }
    
    public void setUI(TabbedPaneUI ui) {
        super.setUI(ui);
        for (int i = 0; i < this.getTabCount(); ++i) {
            if ((((JTabbedPane.Page)this.pages.get(i)).disabledIcon instanceof UIResource)) {
                this.setDisabledIconAt(i, null);
            }
        }
    }
    
    @Override
    public void updateUI() {
        this.setUI((TabbedPaneUI)UIManager.getUI(this));
    }
    
    @Override
    public String getUIClassID() {
        return "TabbedPaneUI";
    }
    
    protected ChangeListener createChangeListener() {
        return new JTabbedPane.ModelListener();
    }
    
    public void addChangeListener(ChangeListener l) {
        this.listenerList.add(ChangeListener.class, l);
    }
    
    public void removeChangeListener(ChangeListener l) {
        this.listenerList.remove(ChangeListener.class, l);
    }
    
    public ChangeListener[] getChangeListeners() {
        return (ChangeListener[])this.listenerList.getListeners(ChangeListener.class);
    }
    
    protected void fireStateChanged() {
        int selectedIndex = this.getSelectedIndex();
        if (selectedIndex < 0) {
            if (this.visComp != null && this.visComp.isVisible()) {
                this.visComp.setVisible(false);
            }
            this.visComp = null;
        }
        else {
            Component component = this.getComponentAt(selectedIndex);
            if (component != null && component != this.visComp) {
                int n = 0;
                if (this.visComp != null) {
                    n = (SwingUtilities.findFocusOwner(this.visComp) != null);
                    if (this.visComp.isVisible()) {
                        this.visComp.setVisible(false);
                    }
                }
                if (!component.isVisible()) {
                    component.setVisible(true);
                }
                if (n != 0) {
                    SwingUtilities2.tabbedPaneChangeFocusTo(component);
                }
                this.visComp = component;
            }
        }
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == ChangeListener.class) {
                if (this.changeEvent == null) {
                    this.changeEvent = new ChangeEvent(this);
                }
                ((ChangeListener)listenerList[i + 1]).stateChanged(this.changeEvent);
            }
        }
    }
    
    public SingleSelectionModel getModel() {
        return this.model;
    }
    
    public void setModel(SingleSelectionModel singleSelectionModel) {
        SingleSelectionModel model = this.getModel();
        if (model != null) {
            model.removeChangeListener(this.changeListener);
            this.changeListener = null;
        }
        if ((this.model = singleSelectionModel) != null) {
            singleSelectionModel.addChangeListener(this.changeListener = this.createChangeListener());
        }
        this.firePropertyChange("model", model, singleSelectionModel);
        this.repaint();
    }
    
    public int getTabPlacement() {
        return this.tabPlacement;
    }
    
    public void setTabPlacement(int tabPlacement) {
        if (tabPlacement != 1 && tabPlacement != 2 && tabPlacement != 3 && tabPlacement != 4) {
            throw new IllegalArgumentException("illegal tab placement: must be TOP, BOTTOM, LEFT, or RIGHT");
        }
        if (this.tabPlacement != tabPlacement) {
            this.firePropertyChange("tabPlacement", this.tabPlacement, this.tabPlacement = tabPlacement);
            this.revalidate();
            this.repaint();
        }
        return;
    }
    
    public int getTabLayoutPolicy() {
        return this.tabLayoutPolicy;
    }
    
    public void setTabLayoutPolicy(int tabLayoutPolicy) {
        if (tabLayoutPolicy != 0 && tabLayoutPolicy != 1) {
            throw new IllegalArgumentException("illegal tab layout policy: must be WRAP_TAB_LAYOUT or SCROLL_TAB_LAYOUT");
        }
        if (this.tabLayoutPolicy != tabLayoutPolicy) {
            this.firePropertyChange("tabLayoutPolicy", this.tabLayoutPolicy, this.tabLayoutPolicy = tabLayoutPolicy);
            this.revalidate();
            this.repaint();
        }
        return;
    }
    
    @Transient
    public int getSelectedIndex() {
        return this.model.getSelectedIndex();
    }
    
    public void setSelectedIndex(int n) {
        if (n != -1) {
            this.checkIndex(n);
        }
        this.setSelectedIndexImpl(n, true);
    }
    
    private void setSelectedIndexImpl(int selectedIndex, boolean b) {
        int selectedIndex2 = this.model.getSelectedIndex();
        JTabbedPane.Page page = null;
        JTabbedPane.Page page2 = null;
        String accessibleName = null;
        int n = b && selectedIndex2 != selectedIndex;
        if (!(n == 0)) {
            if (this.accessibleContext != null) {
                accessibleName = this.accessibleContext.getAccessibleName();
            }
            if (selectedIndex2 >= 0) {
                page = (JTabbedPane.Page)this.pages.get(selectedIndex2);
            }
            if (selectedIndex >= 0) {
                page2 = (JTabbedPane.Page)this.pages.get(selectedIndex);
            }
        }
        this.model.setSelectedIndex(selectedIndex);
        if (!(n == 0)) {
            this.changeAccessibleSelection(page, accessibleName, page2);
        }
    }
    
    private void changeAccessibleSelection(JTabbedPane.Page page, String oldValue, JTabbedPane.Page page2) {
        if (this.accessibleContext == null) {
            return;
        }
        if (page != null) {
            page.firePropertyChange("AccessibleState", AccessibleState.SELECTED, null);
        }
        if (page2 != null) {
            page2.firePropertyChange("AccessibleState", null, AccessibleState.SELECTED);
        }
        this.accessibleContext.firePropertyChange("AccessibleName", oldValue, this.accessibleContext.getAccessibleName());
        return;
    }
    
    @Transient
    public Component getSelectedComponent() {
        int selectedIndex = this.getSelectedIndex();
        if (selectedIndex == -1) {
            return null;
        }
        return this.getComponentAt(selectedIndex);
    }
    
    public void setSelectedComponent(Component component) {
        int indexOfComponent = this.indexOfComponent(component);
        if (indexOfComponent != -1) {
            this.setSelectedIndex(indexOfComponent);
            return;
        }
        throw new IllegalArgumentException("component not found in tabbed pane");
    }
    
    public void insertTab(String s, Icon icon, Component component, String s2, int n) {
        int indexOfComponent = this.indexOfComponent(component);
        if (component != null && indexOfComponent != -1) {
            this.removeTabAt(indexOfComponent);
            if (n > indexOfComponent) {
                --n;
            }
        }
        int selectedIndex = this.getSelectedIndex();
        this.pages.add(n, new JTabbedPane.Page(this, (s != null) ? s : "", icon, null, component, s2));
        if (component != null) {
            this.addImpl(component, null, -1);
            component.setVisible(false);
        }
        else {
            this.firePropertyChange("indexForNullComponent", -1, n);
        }
        if (this.pages.size() == 1) {
            this.setSelectedIndex(0);
        }
        if (selectedIndex >= n) {
            this.setSelectedIndexImpl(selectedIndex + 1, false);
        }
        if (!this.haveRegistered && s2 != null) {
            ToolTipManager.sharedInstance().registerComponent(this);
            this.haveRegistered = true;
        }
        if (this.accessibleContext != null) {
            this.accessibleContext.firePropertyChange("AccessibleVisibleData", null, component);
        }
        this.revalidate();
        this.repaint();
    }
    
    public void addTab(String s, Icon icon, Component component, String s2) {
        this.insertTab(s, icon, component, s2, this.pages.size());
    }
    
    public void addTab(String s, Icon icon, Component component) {
        this.insertTab(s, icon, component, null, this.pages.size());
    }
    
    public void addTab(String s, Component component) {
        this.insertTab(s, null, component, null, this.pages.size());
    }
    
    @Override
    public Component add(Component comp) {
        if (!(comp instanceof UIResource)) {
            this.addTab(comp.getName(), comp);
            return comp;
        }
        super.add(comp);
        return comp;
    }
    
    @Override
    public Component add(String name, Component comp) {
        if (!(comp instanceof UIResource)) {
            this.addTab(name, comp);
            return comp;
        }
        super.add(name, comp);
        return comp;
    }
    
    @Override
    public Component add(Component comp, int index) {
        if (!(comp instanceof UIResource)) {
            this.insertTab(comp.getName(), null, comp, null, (index == -1) ? this.getTabCount() : index);
            return comp;
        }
        super.add(comp, index);
        return comp;
    }
    
    @Override
    public void add(Component comp, Object constraints) {
        if (!(comp instanceof UIResource)) {
            if ((constraints instanceof String)) {
                this.addTab((String)constraints, comp);
            }
            else if ((constraints instanceof Icon)) {
                this.addTab(null, (Icon)constraints, comp);
            }
            else {
                this.add(comp);
            }
        }
        else {
            super.add(comp, constraints);
        }
    }
    
    @Override
    public void add(Component comp, Object constraints, int index) {
        if (!(comp instanceof UIResource)) {
            this.insertTab((constraints instanceof String) ? ((String)constraints) : null, (constraints instanceof Icon) ? ((Icon)constraints) : null, comp, null, (index == -1) ? this.getTabCount() : index);
        }
        else {
            super.add(comp, constraints, index);
        }
    }
    
    public void removeTabAt(int i) {
        this.checkIndex(i);
        Component component = this.getComponentAt(i);
        int n = 0;
        int selectedIndex = this.getSelectedIndex();
        String accessibleName = null;
        if (component == this.visComp) {
            n = (SwingUtilities.findFocusOwner(this.visComp) != null);
            this.visComp = null;
        }
        if (this.accessibleContext != null) {
            if (i == selectedIndex) {
                ((JTabbedPane.Page)this.pages.get(i)).firePropertyChange("AccessibleState", AccessibleState.SELECTED, null);
                accessibleName = this.accessibleContext.getAccessibleName();
            }
            this.accessibleContext.firePropertyChange("AccessibleVisibleData", component, null);
        }
        this.setTabComponentAt(i, null);
        this.pages.remove(i);
        this.putClientProperty("__index_to_remove__", Integer.valueOf(i));
        if (selectedIndex > i) {
            this.setSelectedIndexImpl(selectedIndex - 1, false);
        }
        else if (selectedIndex >= this.getTabCount()) {
            this.setSelectedIndexImpl(selectedIndex - 1, false);
            this.changeAccessibleSelection(null, accessibleName, (selectedIndex != 0) ? ((JTabbedPane.Page)this.pages.get(selectedIndex - 1)) : null);
        }
        else if (i == selectedIndex) {
            this.fireStateChanged();
            this.changeAccessibleSelection(null, accessibleName, (JTabbedPane.Page)this.pages.get(i));
        }
        if (component != null) {
            Component[] components = this.getComponents();
            int length = components.length;
            while (--length >= 0) {
                if (components[length] != component) {
                    continue;
                }
                super.remove(length);
                component.setVisible(true);
                break;
            }
        }
        if (n != 0) {
            SwingUtilities2.tabbedPaneChangeFocusTo(this.getSelectedComponent());
        }
        this.revalidate();
        this.repaint();
    }
    
    @Override
    public void remove(Component component) {
        int indexOfComponent = this.indexOfComponent(component);
        if (indexOfComponent != -1) {
            this.removeTabAt(indexOfComponent);
        }
        else {
            Component[] components = this.getComponents();
            int i = 0;
            while (i < components.length) {
                if (component == components[i]) {
                    super.remove(i);
                    break;
                }
                ++i;
            }
        }
    }
    
    @Override
    public void remove(int n) {
        this.removeTabAt(n);
    }
    
    @Override
    public void removeAll() {
        this.setSelectedIndexImpl(-1, true);
        int tabCount = this.getTabCount();
        while (tabCount-- > 0) {
            this.removeTabAt(tabCount);
        }
    }
    
    public int getTabCount() {
        return this.pages.size();
    }
    
    public int getTabRunCount() {
        if (this.ui != null) {
            return ((TabbedPaneUI)this.ui).getTabRunCount(this);
        }
        return 0;
    }
    
    public String getTitleAt(int n) {
        return ((JTabbedPane.Page)this.pages.get(n)).title;
    }
    
    public Icon getIconAt(int n) {
        return ((JTabbedPane.Page)this.pages.get(n)).icon;
    }
    
    public Icon getDisabledIconAt(int n) {
        JTabbedPane.Page page = (JTabbedPane.Page)this.pages.get(n);
        if (page.disabledIcon == null) {
            page.disabledIcon = UIManager.getLookAndFeel().getDisabledIcon(this, page.icon);
        }
        return page.disabledIcon;
    }
    
    public String getToolTipTextAt(int n) {
        return ((JTabbedPane.Page)this.pages.get(n)).tip;
    }
    
    public Color getBackgroundAt(int n) {
        return ((JTabbedPane.Page)this.pages.get(n)).getBackground();
    }
    
    public Color getForegroundAt(int n) {
        return ((JTabbedPane.Page)this.pages.get(n)).getForeground();
    }
    
    public boolean isEnabledAt(int n) {
        return ((JTabbedPane.Page)this.pages.get(n)).isEnabled();
    }
    
    public Component getComponentAt(int n) {
        return ((JTabbedPane.Page)this.pages.get(n)).component;
    }
    
    public int getMnemonicAt(int n) {
        this.checkIndex(n);
        return ((JTabbedPane.Page)this.pages.get(n)).getMnemonic();
    }
    
    public int getDisplayedMnemonicIndexAt(int n) {
        this.checkIndex(n);
        return ((JTabbedPane.Page)this.pages.get(n)).getDisplayedMnemonicIndex();
    }
    
    public Rectangle getBoundsAt(int n) {
        this.checkIndex(n);
        if (this.ui != null) {
            return ((TabbedPaneUI)this.ui).getTabBounds(this, n);
        }
        return null;
    }
    
    public void setTitleAt(int n, String s) {
        JTabbedPane.Page page = (JTabbedPane.Page)this.pages.get(n);
        String title = page.title;
        page.title = s;
        if (title != s) {
            this.firePropertyChange("indexForTitle", -1, n);
        }
        page.updateDisplayedMnemonicIndex();
        if (title != s && this.accessibleContext != null) {
            this.accessibleContext.firePropertyChange("AccessibleVisibleData", title, s);
        }
        if (s == null || title == null || !s.equals(title)) {
            this.revalidate();
            this.repaint();
        }
    }
    
    public void setIconAt(int n, Icon icon) {
        JTabbedPane.Page page = (JTabbedPane.Page)this.pages.get(n);
        Icon icon2 = page.icon;
        if (icon != icon2) {
            page.icon = icon;
            if ((page.disabledIcon instanceof UIResource)) {
                page.disabledIcon = null;
            }
            if (this.accessibleContext != null) {
                this.accessibleContext.firePropertyChange("AccessibleVisibleData", icon2, icon);
            }
            this.revalidate();
            this.repaint();
        }
    }
    
    public void setDisabledIconAt(int n, Icon disabledIcon) {
        Icon disabledIcon2 = ((JTabbedPane.Page)this.pages.get(n)).disabledIcon;
        ((JTabbedPane.Page)this.pages.get(n)).disabledIcon = disabledIcon;
        if (disabledIcon != disabledIcon2 && !this.isEnabledAt(n)) {
            this.revalidate();
            this.repaint();
        }
    }
    
    public void setToolTipTextAt(int n, String s) {
        String tip = ((JTabbedPane.Page)this.pages.get(n)).tip;
        ((JTabbedPane.Page)this.pages.get(n)).tip = s;
        if (tip != s && this.accessibleContext != null) {
            this.accessibleContext.firePropertyChange("AccessibleVisibleData", tip, s);
        }
        if (!this.haveRegistered && s != null) {
            ToolTipManager.sharedInstance().registerComponent(this);
            this.haveRegistered = true;
        }
    }
    
    public void setBackgroundAt(int n, Color background) {
        Color background2 = ((JTabbedPane.Page)this.pages.get(n)).background;
        ((JTabbedPane.Page)this.pages.get(n)).setBackground(background);
        if (background == null || background2 == null || !background.equals(background2)) {
            Rectangle bounds = this.getBoundsAt(n);
            if (bounds != null) {
                this.repaint(bounds);
            }
        }
    }
    
    public void setForegroundAt(int n, Color foreground) {
        Color foreground2 = ((JTabbedPane.Page)this.pages.get(n)).foreground;
        ((JTabbedPane.Page)this.pages.get(n)).setForeground(foreground);
        if (foreground == null || foreground2 == null || !foreground.equals(foreground2)) {
            Rectangle bounds = this.getBoundsAt(n);
            if (bounds != null) {
                this.repaint(bounds);
            }
        }
    }
    
    public void setEnabledAt(int n, boolean enabled) {
        int enabled2 = ((JTabbedPane.Page)this.pages.get(n)).isEnabled();
        ((JTabbedPane.Page)this.pages.get(n)).setEnabled(enabled);
        if ((enabled ? 1 : 0) != enabled2) {
            this.revalidate();
            this.repaint();
        }
    }
    
    public void setComponentAt(int n, Component component) {
        JTabbedPane.Page page = (JTabbedPane.Page)this.pages.get(n);
        if (component != page.component) {
            int n2 = 0;
            if (page.component != null) {
                n2 = (SwingUtilities.findFocusOwner(page.component) != null);
                synchronized (this.getTreeLock()) {
                    int componentCount = this.getComponentCount();
                    Component[] components = this.getComponents();
                    for (int i = 0; i < componentCount; ++i) {
                        if (components[i] == page.component) {
                            super.remove(i);
                        }
                    }
                }
            }
            page.component = component;
            int visible = this.getSelectedIndex() == n;
            if (!(visible == 0)) {
                this.visComp = component;
            }
            if (component != null) {
                component.setVisible((boolean)visible);
                this.addImpl(component, null, -1);
                if (n2 != 0) {
                    SwingUtilities2.tabbedPaneChangeFocusTo(component);
                }
            }
            else {
                this.repaint();
            }
            this.revalidate();
        }
    }
    
    public void setDisplayedMnemonicIndexAt(int n, int displayedMnemonicIndex) {
        this.checkIndex(n);
        ((JTabbedPane.Page)this.pages.get(n)).setDisplayedMnemonicIndex(displayedMnemonicIndex);
    }
    
    public void setMnemonicAt(int n, int mnemonic) {
        this.checkIndex(n);
        ((JTabbedPane.Page)this.pages.get(n)).setMnemonic(mnemonic);
        this.firePropertyChange("mnemonicAt", null, null);
    }
    
    public int indexOfTab(String s) {
        int i = 0;
        while (i < this.getTabCount()) {
            if (this.getTitleAt(i).equals((s == null) ? "" : s)) {
                return i;
            }
            ++i;
        }
        return -1;
    }
    
    public int indexOfTab(Icon obj) {
        int i = 0;
        while (i < this.getTabCount()) {
            Icon icon = this.getIconAt(i);
            if ((icon != null && icon.equals(obj)) || (icon == null && icon == obj)) {
                return i;
            }
            ++i;
        }
        return -1;
    }
    
    public int indexOfComponent(Component obj) {
        int i = 0;
        while (i < this.getTabCount()) {
            Component component = this.getComponentAt(i);
            if ((component != null && component.equals(obj)) || (component == null && component == obj)) {
                return i;
            }
            ++i;
        }
        return -1;
    }
    
    public int indexAtLocation(int n, int n2) {
        if (this.ui != null) {
            return ((TabbedPaneUI)this.ui).tabForCoordinate(this, n, n2);
        }
        return -1;
    }
    
    @Override
    public String getToolTipText(MouseEvent mouseEvent) {
        if (this.ui != null) {
            int tabForCoordinate = ((TabbedPaneUI)this.ui).tabForCoordinate(this, mouseEvent.getX(), mouseEvent.getY());
            if (tabForCoordinate != -1) {
                return ((JTabbedPane.Page)this.pages.get(tabForCoordinate)).tip;
            }
        }
        return super.getToolTipText(mouseEvent);
    }
    
    private void checkIndex(int i) {
        if (i < 0 || i >= this.pages.size()) {
            throw new IndexOutOfBoundsException("Index: " + i + ", Tab count: " + this.pages.size());
        }
        return;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        if (this.getUIClassID().equals("TabbedPaneUI")) {
            int n = (byte)(JComponent.getWriteObjCounter(this) - 1);
            JComponent.setWriteObjCounter(this, (byte)n);
            if (n == 0 && this.ui != null) {
                this.ui.installUI(this);
            }
        }
    }
    
    @Override
    void compWriteObjectNotify() {
        super.compWriteObjectNotify();
        if (super.getToolTipText() == null && this.haveRegistered) {
            ToolTipManager.sharedInstance().unregisterComponent(this);
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        if (this.ui != null && this.getUIClassID().equals("TabbedPaneUI")) {
            this.ui.installUI(this);
        }
        if (super.getToolTipText() == null && this.haveRegistered) {
            ToolTipManager.sharedInstance().registerComponent(this);
        }
    }
    
    @Override
    protected String paramString() {
        String str;
        if (this.tabPlacement == 1) {
            str = "TOP";
        }
        else if (this.tabPlacement == 3) {
            str = "BOTTOM";
        }
        else if (this.tabPlacement == 2) {
            str = "LEFT";
        }
        else if (this.tabPlacement == 4) {
            str = "RIGHT";
        }
        else {
            str = "";
        }
        return super.paramString() + ",haveRegistered=" + (this.haveRegistered ? "true" : "false") + ",tabPlacement=" + str;
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new JTabbedPane.AccessibleJTabbedPane();
            for (int tabCount = this.getTabCount(), i = 0; i < tabCount; ++i) {
                ((JTabbedPane.Page)this.pages.get(i)).initAccessibleContext();
            }
        }
        return this.accessibleContext;
    }
    
    public void setTabComponentAt(int n, Component tabComponent) {
        if (tabComponent != null && this.indexOfComponent(tabComponent) != -1) {
            throw new IllegalArgumentException("Component is already added to this JTabbedPane");
        }
        if (tabComponent != this.getTabComponentAt(n)) {
            int indexOfTabComponent = this.indexOfTabComponent(tabComponent);
            if (indexOfTabComponent != -1) {
                this.setTabComponentAt(indexOfTabComponent, null);
            }
            ((JTabbedPane.Page)this.pages.get(n)).tabComponent = tabComponent;
            this.firePropertyChange("indexForTabComponent", -1, n);
        }
        return;
    }
    
    public Component getTabComponentAt(int n) {
        return ((JTabbedPane.Page)this.pages.get(n)).tabComponent;
    }
    
    public int indexOfTabComponent(Component component) {
        int i = 0;
        while (i < this.getTabCount()) {
            if (this.getTabComponentAt(i) == component) {
                return i;
            }
            ++i;
        }
        return -1;
    }
    
    static void access$200(JTabbedPane tabbedPane, String propertyName, Object oldValue, Object newValue) {
        tabbedPane.firePropertyChange(propertyName, oldValue, newValue);
    }
    
    protected class ModelListener implements ChangeListener, Serializable
    {
        @Override
        public void stateChanged(ChangeEvent changeEvent) {
            JTabbedPane.this.fireStateChanged();
        }
    }
    
    protected class AccessibleJTabbedPane extends JComponent.AccessibleJComponent implements AccessibleSelection, ChangeListener
    {
        @Override
        public String getAccessibleName() {
            if (this.accessibleName != null) {
                return this.accessibleName;
            }
            String s = (String)JTabbedPane.this.getClientProperty("AccessibleName");
            if (s != null) {
                return s;
            }
            int selectedIndex = JTabbedPane.this.getSelectedIndex();
            if (selectedIndex >= 0) {
                return ((JTabbedPane.Page)JTabbedPane.this.pages.get(selectedIndex)).getAccessibleName();
            }
            return super.getAccessibleName();
        }
        
        public AccessibleJTabbedPane() {
            JTabbedPane.this.model.addChangeListener(this);
        }
        
        @Override
        public void stateChanged(ChangeEvent changeEvent) {
            this.firePropertyChange("AccessibleSelection", null, changeEvent.getSource());
        }
        
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.PAGE_TAB_LIST;
        }
        
        @Override
        public int getAccessibleChildrenCount() {
            return JTabbedPane.this.getTabCount();
        }
        
        @Override
        public Accessible getAccessibleChild(int n) {
            if (n < 0 || n >= JTabbedPane.this.getTabCount()) {
                return null;
            }
            return (Accessible)JTabbedPane.this.pages.get(n);
        }
        
        @Override
        public AccessibleSelection getAccessibleSelection() {
            return this;
        }
        
        @Override
        public Accessible getAccessibleAt(Point point) {
            int n = ((TabbedPaneUI)JTabbedPane.this.ui).tabForCoordinate(JTabbedPane.this, point.x, point.y);
            if (n == -1) {
                n = JTabbedPane.this.getSelectedIndex();
            }
            return this.getAccessibleChild(n);
        }
        
        @Override
        public int getAccessibleSelectionCount() {
            return 1;
        }
        
        @Override
        public Accessible getAccessibleSelection(int n) {
            int selectedIndex = JTabbedPane.this.getSelectedIndex();
            if (selectedIndex == -1) {
                return null;
            }
            return (Accessible)JTabbedPane.this.pages.get(selectedIndex);
        }
        
        @Override
        public boolean isAccessibleChildSelected(int n) {
            return n == JTabbedPane.this.getSelectedIndex();
        }
        
        @Override
        public void addAccessibleSelection(int selectedIndex) {
            JTabbedPane.this.setSelectedIndex(selectedIndex);
        }
        
        @Override
        public void removeAccessibleSelection(int n) {
        }
        
        @Override
        public void clearAccessibleSelection() {
        }
        
        @Override
        public void selectAllAccessibleSelection() {
        }
    }
    
    private class Page extends AccessibleContext implements Serializable, Accessible, AccessibleComponent
    {
        String title;
        Color background;
        Color foreground;
        Icon icon;
        Icon disabledIcon;
        JTabbedPane parent;
        Component component;
        String tip;
        boolean enabled;
        boolean needsUIUpdate;
        int mnemonic;
        int mnemonicIndex;
        Component tabComponent;
        
        Page(JTabbedPane parent, String title, Icon icon, Icon disabledIcon, Component component, String tip) {
            this.enabled = true;
            this.mnemonic = -1;
            this.mnemonicIndex = -1;
            this.title = title;
            this.icon = icon;
            this.disabledIcon = disabledIcon;
            this.setAccessibleParent(this.parent = parent);
            this.component = component;
            this.tip = tip;
            this.initAccessibleContext();
        }
        
        void initAccessibleContext() {
            if (JTabbedPane.this.accessibleContext != null && this.component instanceof Accessible) {
                AccessibleContext accessibleContext = this.component.getAccessibleContext();
                if (accessibleContext != null) {
                    accessibleContext.setAccessibleParent(this);
                }
            }
        }
        
        void setMnemonic(int mnemonic) {
            this.mnemonic = mnemonic;
            this.updateDisplayedMnemonicIndex();
        }
        
        int getMnemonic() {
            return this.mnemonic;
        }
        
        void setDisplayedMnemonicIndex(int n) {
            if (this.mnemonicIndex != n) {
                if (n != -1 && (this.title == null || n < 0 || n >= this.title.length())) {
                    throw new IllegalArgumentException("Invalid mnemonic index: " + n);
                }
                this.mnemonicIndex = n;
                JTabbedPane.access$200(JTabbedPane.this, "displayedMnemonicIndexAt", null, null);
            }
        }
        
        int getDisplayedMnemonicIndex() {
            return this.mnemonicIndex;
        }
        
        void updateDisplayedMnemonicIndex() {
            this.setDisplayedMnemonicIndex(SwingUtilities.findDisplayedMnemonicIndex(this.title, this.mnemonic));
        }
        
        @Override
        public AccessibleContext getAccessibleContext() {
            return this;
        }
        
        @Override
        public String getAccessibleName() {
            if (this.accessibleName != null) {
                return this.accessibleName;
            }
            if (this.title != null) {
                return this.title;
            }
            return null;
        }
        
        @Override
        public String getAccessibleDescription() {
            if (this.accessibleDescription != null) {
                return this.accessibleDescription;
            }
            if (this.tip != null) {
                return this.tip;
            }
            return null;
        }
        
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.PAGE_TAB;
        }
        
        @Override
        public AccessibleStateSet getAccessibleStateSet() {
            AccessibleStateSet accessibleStateSet = this.parent.getAccessibleContext().getAccessibleStateSet();
            accessibleStateSet.add(AccessibleState.SELECTABLE);
            if (this.parent.indexOfTab(this.title) == this.parent.getSelectedIndex()) {
                accessibleStateSet.add(AccessibleState.SELECTED);
                return accessibleStateSet;
            }
            return accessibleStateSet;
        }
        
        @Override
        public int getAccessibleIndexInParent() {
            return this.parent.indexOfTab(this.title);
        }
        
        @Override
        public int getAccessibleChildrenCount() {
            if ((this.component instanceof Accessible)) {
                return 1;
            }
            return 0;
        }
        
        @Override
        public Accessible getAccessibleChild(int n) {
            if ((this.component instanceof Accessible)) {
                return (Accessible)this.component;
            }
            return null;
        }
        
        @Override
        public Locale getLocale() {
            return this.parent.getLocale();
        }
        
        @Override
        public AccessibleComponent getAccessibleComponent() {
            return this;
        }
        
        @Override
        public Color getBackground() {
            return (this.background != null) ? this.background : this.parent.getBackground();
        }
        
        @Override
        public void setBackground(Color background) {
            this.background = background;
        }
        
        @Override
        public Color getForeground() {
            return (this.foreground != null) ? this.foreground : this.parent.getForeground();
        }
        
        @Override
        public void setForeground(Color foreground) {
            this.foreground = foreground;
        }
        
        @Override
        public Cursor getCursor() {
            return this.parent.getCursor();
        }
        
        @Override
        public void setCursor(Cursor cursor) {
            this.parent.setCursor(cursor);
        }
        
        @Override
        public Font getFont() {
            return this.parent.getFont();
        }
        
        @Override
        public void setFont(Font font) {
            this.parent.setFont(font);
        }
        
        @Override
        public FontMetrics getFontMetrics(Font font) {
            return this.parent.getFontMetrics(font);
        }
        
        @Override
        public boolean isEnabled() {
            return this.enabled;
        }
        
        @Override
        public void setEnabled(boolean enabled) {
            this.enabled = enabled;
        }
        
        @Override
        public boolean isVisible() {
            return this.parent.isVisible();
        }
        
        @Override
        public void setVisible(boolean visible) {
            this.parent.setVisible(visible);
        }
        
        @Override
        public boolean isShowing() {
            return this.parent.isShowing();
        }
        
        @Override
        public boolean contains(Point p) {
            return this.getBounds().contains(p);
        }
        
        @Override
        public Point getLocationOnScreen() {
            Point locationOnScreen = this.parent.getLocationOnScreen();
            Point location = this.getLocation();
            location.translate(locationOnScreen.x, locationOnScreen.y);
            return location;
        }
        
        @Override
        public Point getLocation() {
            Rectangle bounds = this.getBounds();
            return new Point(bounds.x, bounds.y);
        }
        
        @Override
        public void setLocation(Point point) {
        }
        
        @Override
        public Rectangle getBounds() {
            return this.parent.getUI().getTabBounds(this.parent, this.parent.indexOfTab(this.title));
        }
        
        @Override
        public void setBounds(Rectangle rectangle) {
        }
        
        @Override
        public Dimension getSize() {
            Rectangle bounds = this.getBounds();
            return new Dimension(bounds.width, bounds.height);
        }
        
        @Override
        public void setSize(Dimension dimension) {
        }
        
        @Override
        public Accessible getAccessibleAt(Point point) {
            if ((this.component instanceof Accessible)) {
                return (Accessible)this.component;
            }
            return null;
        }
        
        @Override
        public boolean isFocusTraversable() {
            return false;
        }
        
        @Override
        public void requestFocus() {
        }
        
        @Override
        public void addFocusListener(FocusListener focusListener) {
        }
        
        @Override
        public void removeFocusListener(FocusListener focusListener) {
        }
        
        @Override
        public AccessibleIcon[] getAccessibleIcon() {
            AccessibleIcon accessibleIcon = null;
            if (this.enabled && this.icon instanceof ImageIcon) {
                accessibleIcon = (AccessibleIcon)((ImageIcon)this.icon).getAccessibleContext();
            }
            else if (!this.enabled && this.disabledIcon instanceof ImageIcon) {
                accessibleIcon = (AccessibleIcon)((ImageIcon)this.disabledIcon).getAccessibleContext();
            }
            if (accessibleIcon != null) {
                return new AccessibleIcon[] { accessibleIcon };
            }
            return null;
        }
    }
}
